#import "utils.typ": todo, inwriting

= Introduction
== Motivation
Anomaly detection has especially in the industrial and automotive field essential importance.
Lots of assembly lines need visual inspection to find errors often with the help of camera systems.
Machine learning helped the field to advance a lot in the past.
Most of the time the error rate is sub $.1%$ and therefore plenty of good data and almost no faulty data is available.
So the train data is heavily unbalaned.#cite(<parnami2022learningexamplessummaryapproaches>)

PatchCore and EfficientAD are state of the art algorithms trained only on good data and then detect anomalies within unseen (but similar) data.
One of their problems is the need of lots of training data and time to train.
Moreover a slight change of the camera position or the lighting conditions can lead to a complete retraining of the model.
Few-Shot learning might be a suitable alternative with hugely lowered train times and fast adaption to new conditions.~#cite(<efficientADpaper>)#cite(<patchcorepaper>)#cite(<parnami2022learningexamplessummaryapproaches>)

In this thesis the performance of 3 Few-Shot learning algorithms will be compared in the field of anomaly detection.
Moreover, few-shot learning might be able not only to detect anomalies but also to detect the anomaly class.

== Research Questions <sectionresearchquestions>

=== Is Few-Shot learning a suitable fit for anomaly detection?

Should Few-Shot learning be used for anomaly detection tasks?
How does it compare to well established algorithms such as Patchcore or EfficientAD?

=== How does disbalancing the Shot number affect performance?
Does giving the Few-Shot learner more good than bad samples improve the model performance?

=== How does the 3 (ResNet, CAML, \pmf) methods perform in only detecting the anomaly class?
How much does the performance improve if only detecting an anomaly or not?
How does it compare to PatchCore and EfficientAD?

#if inwriting [
=== Extra: How does Euclidean distance compare to Cosine-similarity when using ResNet as a feature-extractor?
// I've tried different distance measures $->$ but results are pretty much the same.
]

== Outline
This thesis is structured to provide a comprehensive exploration of Few-Shot Learning in anomaly detection.
@sectionmaterialandmethods introduces the datasets and methodologies used in this research.
The MVTec AD dataset is discussed in detail as the primary source for benchmarking, along with an overview of the Few-Shot Learning paradigm.
The section elaborates on the three selected methods—ResNet50, P>M>F, and CAML—while also touching upon well established anomaly detection algorithms such as Pachcore and EfficientAD.

@sectionimplementation focuses on the practical realization of the methods described in the previous chapter.
It outlines the experimental setup, including the use of Jupyter Notebook for prototyping and testing, and provides a detailed account of how each method was implemented and evaluated.

The experimental outcomes are presented in @sectionexperimentalresults.
This section addresses the research questions posed in @sectionresearchquestions, examining the suitability of Few-Shot Learning for anomaly detection tasks, the impact of class imbalance on model performance, and the comparative effectiveness of the three selected methods.
Additional experiments explore the differences between Euclidean distance and Cosine similarity when using ResNet as a feature extractor.#todo[Maybe remove this]

Finally, @sectionconclusionandoutlook, summarizes the key findings of this study.
It reflects on the implications of the results for the field of anomaly detection and proposes directions for future research that could address the limitations and enhance the applicability of Few-Shot Learning approaches in this domain.
